#include <stdlib.h>
#include <math.h>

#include "EnvelopeParams.h"
#include "LFOParams.h"
#include "FilterParams.h"
#include "ADnoteParameters.h"


ADnoteParameters::ADnoteParameters(){
    int nvoice;
    //Default Parameters
    /* Parametrii Globali de Frecventa */
    GlobalPar.PDetune=8192;//zero
    GlobalPar.FreqEnvelope=new EnvelopeParams();
    GlobalPar.FreqEnvelope->ASRinit(64,80,64,120);
    GlobalPar.FreqLfo=new LFOParams(70,10,0,0,0,20,0);
    
    /* Parametrii Globali de Amplitudine */
    GlobalPar.PVolume=127;
    GlobalPar.PAmpVelocityScaleFunction=64;
    GlobalPar.AmpEnvelope=new EnvelopeParams();
    GlobalPar.AmpEnvelope->ADSRinit_dB(80,1,127,20);
    GlobalPar.AmpLfo=new LFOParams(60,0,0,0,0,30,1);
    
    /* Parametrii Globali de Filtru */
    GlobalPar.PFilterVelocityScale=64;
    GlobalPar.PFilterVelocityScaleFunction=64;
    GlobalPar.GlobalFilter=new FilterParams(2,70,90);
    GlobalPar.FilterEnvelope=new EnvelopeParams();
    GlobalPar.FilterEnvelope->ADSRinit_filter(40,80,90,100,100,40);
    GlobalPar.FilterLfo=new LFOParams(80,0,0,0,0,30,2);


    for (nvoice=0;nvoice<NUM_VOICES;nvoice++){
	VoicePar[nvoice].Enabled=ON;

	VoicePar[nvoice].PVolume=127;
	VoicePar[nvoice].PDetune=8192;//8192=0
	VoicePar[nvoice].PFreqLfoEnabled=1;
	VoicePar[nvoice].PFreqEnvelopeEnabled=0;
	VoicePar[nvoice].PAmpEnvelopeEnabled=0;
	VoicePar[nvoice].PAmpLfoEnabled=0;
	VoicePar[nvoice].PAmpVelocityScaleFunction=127;//Altfel la velocity-uri mici nu suna aproape deloc
	VoicePar[nvoice].PFilterEnabled=0;
	VoicePar[nvoice].PFilterEnvelopeEnabled=0;
	VoicePar[nvoice].PFilterLfoEnabled=0;
	VoicePar[nvoice].PFMEnabled=0;
	VoicePar[nvoice].PFMVolume=90;
	VoicePar[nvoice].PFMDetune=8192;
	VoicePar[nvoice].PFMFreqEnvelopeEnabled=0;
	VoicePar[nvoice].PFMAmpEnvelopeEnabled=0;
	VoicePar[nvoice].PFMVelocityScaleFunction=64;	
	
	EnableVoice(nvoice);
    };

/*test*/
    for (nvoice=0;nvoice<NUM_VOICES;nvoice++){
	VoicePar[nvoice].PDetune=8192+(int)(RND*2500.0);
    };
    
/*test*/    
    
    midi2real();
};

void ADnoteParameters::EnableVoice(int nvoice){
    int k;
    VoicePar[nvoice].OscilSmp=new REALTYPE[OSCIL_SIZE];
    VoicePar[nvoice].FMSmp=new REALTYPE[OSCIL_SIZE];

    //temporar - test
//    for (k=0;k<OSCIL_SIZE;k++) VoicePar[nvoice].OscilSmp[k]=sin(PI*(REALTYPE) k/(REALTYPE) OSCIL_SIZE*2.0);
    for (k=0;k<OSCIL_SIZE;k++) VoicePar[nvoice].OscilSmp[k]=pow(sin(PI*(REALTYPE) k/(REALTYPE) OSCIL_SIZE+1.5),25);
    for (k=0;k<OSCIL_SIZE;k++) VoicePar[nvoice].FMSmp[k]=sin(PI*(REALTYPE) k/(REALTYPE) OSCIL_SIZE*2.0);
    
    VoicePar[nvoice].AmpEnvelope=new EnvelopeParams();
    VoicePar[nvoice].AmpEnvelope->ADSRinit_dB(4,100,127,100);	
    VoicePar[nvoice].AmpLfo=new LFOParams(90,32,0,0,0,30,1);;

    VoicePar[nvoice].FreqEnvelope=new EnvelopeParams();
    VoicePar[nvoice].FreqEnvelope->ASRinit(64,50,64,60);
    VoicePar[nvoice].FreqLfo=new LFOParams(30+(int) (50*RND),32,(int)(127*RND),0,0,0,0);

    VoicePar[nvoice].VoiceFilter=new FilterParams(0,80,10);
    VoicePar[nvoice].FilterEnvelope=new EnvelopeParams();
    VoicePar[nvoice].FilterEnvelope->ADSRinit_filter(90,70,40,70,10,40);
    VoicePar[nvoice].FilterLfo=new LFOParams(64,32,0,0,0,30,2);

    VoicePar[nvoice].FMFreqEnvelope=new EnvelopeParams();
    VoicePar[nvoice].FMFreqEnvelope->ASRinit(110,70,40,80);    
    VoicePar[nvoice].FMAmpEnvelope=new EnvelopeParams();
    VoicePar[nvoice].FMAmpEnvelope->ADSRinit(100,40,64,100);                
};

REALTYPE ADnoteParameters::getdetune(unsigned short int midipar){
    REALTYPE realpar;
    if (midipar>16384) midipar=16384;
    realpar=exp(fabs((float)(midipar-8192))/8192.0*LOG_2*12)-1;
    if (midipar<8192) realpar=-realpar;
    return(realpar);
};

void ADnoteParameters::midi2real(){
    int i;
    GlobalPar.Detune=getdetune(GlobalPar.PDetune);
    GlobalPar.Volume=(REALTYPE) GlobalPar.PVolume/127.0;
    GlobalPar.AmpVelocityScaleFunction=((REALTYPE) GlobalPar.PAmpVelocityScaleFunction-64.0)/64.0;
    GlobalPar.FilterVelocityScale=(REALTYPE) GlobalPar.PFilterVelocityScale/127.0*5.0;
    GlobalPar.FilterVelocityScaleFunction=((REALTYPE) GlobalPar.PFilterVelocityScaleFunction-64.0)/64.0;
    for (i=0;i<NUM_VOICES;i++){
	if (VoicePar[i].Enabled==OFF) continue;
	VoicePar[i].Detune=getdetune(VoicePar[i].PDetune);
	VoicePar[i].Volume=(REALTYPE) VoicePar[i].PVolume/127.0;
	VoicePar[i].AmpVelocityScaleFunction=((REALTYPE) VoicePar[i].PAmpVelocityScaleFunction-64.0)/64.0;
    	switch (VoicePar[i].PFMEnabled){
	    case 1:VoicePar[i].FMEnabled=FREQ_MOD; break;
	    case 2:VoicePar[i].FMEnabled=RING_MOD; break;
	    default:VoicePar[i].FMEnabled=NONE;
	};
	VoicePar[i].FMVolume=(REALTYPE) VoicePar[i].PFMVolume/127.0;
	VoicePar[i].FMVelocityScaleFunction=((REALTYPE) VoicePar[i].PFMVelocityScaleFunction-64.0)/64.0;
	VoicePar[i].FMDetune=getdetune(VoicePar[i].PFMDetune);			
    };
        
};

ADnoteParameters::~ADnoteParameters(){
};

