#include "../globals.h"
#include "Envelope.h"
#include "LFO.h"
#include "Filter.h"
#include "../Params/ADnoteParameters.h"

//Globals

//marimea unei forme de unda (la oscil) - putere al lui 2 !!!!
#define OSCIL_SIZE 512

//reglajul intensitatii FM
#define FM_AMP_MULTIPLIER 1000000.0

//felul maxim in care velocity-ul este scalat f(vel,a)=vel^a
#define VELOCITY_MAX_SCALE 10.0

class ADnote{   //ADditive voice
 public:
  ADnote(ADnoteParameters *pars,int note,REALTYPE velocity);  
  ~ADnote();
  int noteout(REALTYPE *out); //trimite in tabloul *out sunetul de marimea sndbufsize
  void relasekey();//trimite cand am lasat degetul de pe clapa
  int finished();
 private:
    
  void setfreq(int nvoice,REALTYPE freq);
  void setfreqFM(int nvoice,REALTYPE freq);
  void computecurrentparameters();
  void initparameters();
  void KillVoice(int nvoice);
  void KillNote();
  inline void ComputeVoiceOscillator(int nvoice);
  inline void ComputeVoiceOscillatorRingModulation(int nvoice);
  inline void ComputeVoiceOscillatorFM(int nvoice);

  //transforma velocity-ul in functie de un scaling
  inline REALTYPE VelF(REALTYPE scaling);

  //GLOBAL
    ADnoteParameters partparams;
    int note;
    REALTYPE velocity;    

    //La inceputul notei este ON, dupa terminarea ei este OFF
    ONOFFTYPE NoteEnabled;

   /*****************************************************************/
   /*                    PARAMETRII GLOBALI                         */
   /*****************************************************************/

    struct ADnoteGlobal{
       /******************************************
	*     PARAMETRII GLOBALI DE FRECVENTA    *
	******************************************/
	REALTYPE Detune;//se masoara in centi

	Envelope *FreqEnvelope; //"Envelop-ul" frecventei (global)
    
	LFO *FreqLfo;//LFO-ul global al frecventei

       /********************************************
	*     PARAMETRII GLOBALI DE AMPLITUDINE    *
	********************************************/
	REALTYPE Volume;// [ 0 .. 1 ]

	REALTYPE AmpVelocityScaleFunction;//[ -1 .. 1 ]

	Envelope *AmpEnvelope;
   
	LFO *AmpLfo;   

       /******************************************
	*     PARAMETRII GLOBALI DE FILTRU       *
	******************************************/
	Filter *GlobalFilter;

	//parametru care depinde de "velocity"
	REALTYPE FilterVelocityScale; //cate octave
	//parametru care depinde de "velocity"
	REALTYPE FilterVelocityScaleFunction;//[ -1 .. 1 ]
    
	REALTYPE FilterCenterPitch;//normal este zero,se masoara in octave
    
	Envelope *FilterEnvelope;
    
	LFO *FilterLfo;
    } NoteGlobalPar;  


    
   /***********************************************************************/
   /*                    PARAMETRII FIECAREI VOCI                         */
   /***********************************************************************/
    struct ADnoteVoice{
        /*Daca este activata Vocea*/
        ONOFFTYPE Enabled; 
  
        /*Forma de unda al oscilatorului din Voce*/ 
        REALTYPE *OscilSmp;    

        /**********************************
	*     PARAMETRII DE FRECVENTA    *
	**********************************/

	/*Reglajul 'fin' a frecventei vocii*/
	REALTYPE Detune;//(se masoara in centi)frecventa de baza a fiecarui oscilator = basefreq*VoiceDetune
    
	/*Envelop-ul frecventei pt. fiecare voce (glissando)*/
	ONOFFTYPE FreqEnvelopeEnabled;
	Envelope *FreqEnvelope;

	/*LFO-ul frecventei pentru fiecare voce (vibratto)*/
	ONOFFTYPE FreqLfoEnabled;
	LFO *FreqLfo;
    

	/*********************************
	*   PARAMETRII DE AMPLITUDINE   *
	*********************************/

	/*Volumul vocii*/          
	REALTYPE Volume;

	/* Functia de transformare al velocity-ului*/
	REALTYPE AmpVelocityScaleFunction;//[ -1 .. 1 ]

	/*Envelop-ul amplitudinii pt. fiecare voce (ADSR)*/
	ONOFFTYPE AmpEnvelopeEnabled;
	Envelope *AmpEnvelope;
    
	/*LFO-ul amplitudinii pt. fiecare voce (termollo)*/
	ONOFFTYPE AmpLfoEnabled;
	LFO *AmpLfo;



	/****************************
	*   PARAMETRII DE FILTRU   *
	****************************/
    
	/*Filtrul pentru fiecare voce*/
	ONOFFTYPE FilterEnabled;
	Filter *VoiceFilter;
    
	/*centrul frecventei filtrului (depinde de Velocity)*/
	REALTYPE FilterCenterPitch;//normal este zero

	/*Envelope-ul filtrului pentru fiecare voce*/
	ONOFFTYPE FilterEnvelopeEnabled;
	Envelope *FilterEnvelope;
    
	/*LFO-ul filtrului pentru fiecare voce*/
	ONOFFTYPE FilterLfoEnabled;
	LFO *FilterLfo;

	/********************************
	*   PARAMETRII DE FM/RingMod   *
	********************************/

	/*Daca este activata FM-ul sau RingModulation*/
	FMTYPE FMEnabled;

	/*Forma de unda al oscilatorului din Voce*/
	REALTYPE *FMSmp;    

	/*Volumul (intensitatea) FM-ului/RM*/
	REALTYPE FMVolume;

	/* Functia de transformare al velocity-ului*/
	REALTYPE FMVelocityScaleFunction;//[ -1 .. 1 ]

	/*Acordul fin al FM-ului/RM*/
	REALTYPE FMDetune; //in centi
    
	/*Envelope-ul FRECVENTEI FM-ului/RM  pentru fiecare voce*/
	ONOFFTYPE FMFreqEnvelopeEnabled;
	Envelope *FMFreqEnvelope;

	/*Envelope-ul Amplitudinii FM-ului/RM  pentru fiecare voce*/
	ONOFFTYPE FMAmpEnvelopeEnabled;
	Envelope *FMAmpEnvelope;
    } NoteVoicePar[NUM_VOICES]; 


   /***********************************************************************/
   /*             VALORILE INTERNE ALE NOTEI SI ALE VOCILOR               */
   /***********************************************************************/

    REALTYPE time;//durata de cand suna nota
    REALTYPE oscposlo[NUM_VOICES],oscfreqlo[NUM_VOICES];    //partea factionara a pozitiei,skip(freq)
    short int oscposhi[NUM_VOICES],oscfreqhi[NUM_VOICES];//partea intreaga a pozitiei,skip(freq)

    REALTYPE oscposloFM[NUM_VOICES],oscfreqloFM[NUM_VOICES];    //partea factionara a pozitiei,skip(freq)
    unsigned short int oscposhiFM[NUM_VOICES],oscfreqhiFM[NUM_VOICES];//partea intreaga a pozitiei,skip(freq)

    REALTYPE oldamplitude[NUM_VOICES];//necesari pentru a calcula amplitudinile
    REALTYPE newamplitude[NUM_VOICES];//necesari pentru a calcula amplitudinile

    REALTYPE FMoldamplitude[NUM_VOICES],FMnewamplitude[NUM_VOICES];//necesari pentru a calcula amplitudinile FM
    
    REALTYPE tmpwave[SOUND_BUFFER_SIZE];//calculez mai intai in bufferul asta
    
    REALTYPE globaloldamplitude,globalnewamplitude;
};

