#include "Filter.h"
#include <math.h>

#include <stdio.h>

Filter::Filter(FilterParams *filterpars){
    type=filterpars->Ptype;
    freq=1000.0;
    q=filterpars->q;
    x1=0.0;x2=0.0;y1=0;y2=0;
};

Filter::~Filter(){
};

void Filter::computefiltercoefs(){
    REALTYPE tmp;
    REALTYPE omega,sn,cs,alpha;
    //do not allow bogus freqs
    if (freq>(SAMPLE_RATE/2-100)) freq=SAMPLE_RATE/2-100;
    if (freq<0.1) freq=0.1;
    //do not allow bogus Q
    if (q<0.0) q=0.0;
    
    if (type==0){
	tmp=exp(-2.0*PI*freq/SAMPLE_RATE);
	c0=1.0-tmp;c1=0.0;
	d1=tmp;
	order=1;
	};
    if (type==2){
	omega=2*PI*freq/SAMPLE_RATE;
//	fprintf(stderr,"%f ",omega);
	sn=sin(omega);
	cs=cos(omega);
	alpha=sn/(2*q);
	tmp=1+alpha;
	c0=(1.0-cs)/2.0/tmp;
	c1=1.0-cs/tmp;
	c2=c0/tmp;
	d1=-(-2*cs)/tmp;
	d2=-(1-alpha)/tmp;
	order=2;
    };
};


void Filter::setfreq(REALTYPE frequency){
    freq=frequency;
    computefiltercoefs();
};



void Filter::filterout(REALTYPE *smp){
    int i;
    REALTYPE y0;
    //Filtru de ordin 1
    if (order==1) {
	for (i=0;i<SOUND_BUFFER_SIZE;i++){
	    y0=smp[i]*c0+x1*c1+y1*d1;
	    y1=y0;
	    x1=smp[i];
	    //output
	    smp[i]=y0;
	};
    };
    if (order==2) {
	for (i=0;i<SOUND_BUFFER_SIZE;i++){
	    y0=smp[i]*c0+x1*c1+x2*c2+y1*d1+y2*d2;
	    y2=y1;
	    y1=y0;
	    x2=x1;
	    x1=smp[i];
	    //output
	    smp[i]=y0;
	};
    };
};

