#include <stdlib.h>
#include <math.h>

#include "LFO.h"


LFO::LFO(LFOParams *lfopars){
    x=lfopars->startphase-floor(lfopars->startphase);
    incx=fabs(lfopars->freq)*(REALTYPE)SOUND_BUFFER_SIZE/(REALTYPE)SAMPLE_RATE;

    //Limit the Frequency(or else...)
    if (incx>0.49999999) incx=0.499999999;
    

    lfornd=lfopars->randomness;
    if (lfornd<0.0) lfornd=0.0;
	else if (lfornd>1.0) lfornd=1.0;

    lfointensity=lfopars->intensity;
    amp1=(1-lfornd)+lfornd*RND;
    amp2=(1-lfornd)+lfornd*RND;
    lfotype=lfopars->PLFOtype;
    lfodelay=lfopars->delay/1000.0;
};

LFO::~LFO(){
};

REALTYPE LFO::lfoout(){
    REALTYPE out;    
    switch (lfotype){
	case 1: //LFO_TRIANGLE
		if ((x>0.0)&&(x<0.25)) out=4.0*x;
		else if ((x>0.25)&&(x<0.75)) out=2-4*x;
		     else out=4.0*x-4.0;
		break;
	case 2: //LFO_SQUARE
		if (x<0.5) out=-1; 
		    else   out=1;
		break;
	case 3: //LFO_RAMPUP
		out=(x-0.5)*2.0;
		break;
	case 4: //LFO_RAMPDOWN
		out=(0.5-x)*2.0;
		break;
	default:out=cos(x*2*PI);//LFO_SINE
    };
     if ((lfotype==0)||(lfotype==1)) out*=lfointensity*(amp1+x*(amp2-amp1));
        else out*=lfointensity*amp2;
    if (lfodelay<0.00001) {
	    x+=incx;
	    if (x>1) {
		x-=1;
		amp1=amp2;
		amp2=(1-lfornd)+lfornd*RND;
		};
	    }
	else lfodelay-=(REALTYPE)SOUND_BUFFER_SIZE/(REALTYPE)SAMPLE_RATE;
    
    
    
    return(out);
};

REALTYPE LFO::amplfoout(){
    REALTYPE out;
    out=1-lfointensity+lfoout()/2.0;
    if (out<-1.0) out=-1.0;
	else if (out>1.0) out=1.0;
    return(out);
};

